﻿using System;
using System.Globalization;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Routing;

namespace HowDoISamples
{
    public class GetShortestPathByCoordinates : UserControl
    {
        public GetShortestPathByCoordinates()
        {
            InitializeComponent();
        }

        private void UserControl_Load(object sender, EventArgs e)
        {
            RenderMap();
        }

        private void btnRoute_Click(object sender, EventArgs e)
        {
            RoutingSource routingSource = new RtgRoutingSource(@"...\rami1\Build\JD_SAMPLE.rtg");
            ShapeFileFeatureSource featureSource = new ShapeFileFeatureSource(@"...\rami1\Build\JD_SAMPLE.shp");
            RoutingEngine routingEngine = new RoutingEngine(routingSource, featureSource);
            RoutingLayer routingLayer = (RoutingLayer)((LayerOverlay)winformsMap1.Overlays["RoutingOverlay"]).Layers["RoutingLayer"];

            routingSource.Open();
            featureSource.Open();
            string startFeatureId = featureSource.GetFeaturesNearestTo(routingLayer.StartPoint, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.NoColumns)[0].Id;
            string endFeatureId = featureSource.GetFeaturesNearestTo(routingLayer.EndPoint, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.NoColumns)[0].Id;
            string startSegmentId = routingSource.GetRouteSegmentByFeatureId(startFeatureId).FeatureId;
            string endSegmentId = routingSource.GetRouteSegmentByFeatureId(endFeatureId).FeatureId;

            RoutingResult result = new RoutingResult();
            if (startSegmentId == endSegmentId)
            {
                Feature feature = featureSource.GetFeatureById(startFeatureId, new string[] { "TRAVEL_DIR"});
                LineShape line = ((MultilineShape)feature.GetShape()).Lines[0];
                double startDistance = routingLayer.StartPoint.GetDistanceTo(new PointShape(line.Vertices[0].X, line.Vertices[0].Y), GeographyUnit.DecimalDegree, DistanceUnit.Meter);
                double endDistance = routingLayer.EndPoint.GetDistanceTo(new PointShape(line.Vertices[0].X, line.Vertices[0].Y), GeographyUnit.DecimalDegree, DistanceUnit.Meter);
                // if the one-way direction is from start to end
                if (feature.ColumnValues["TRAVEL_DIR"] == "1")
                {
                    if (startDistance < endDistance)
                    {
                        result.Features.Add(new Feature(line));
                        result.Route.Lines.Add(line);
                    }
                }
                else
                {
                    result = routingEngine.GetRoute(startFeatureId, endFeatureId);
                }
            }

            routingLayer.Routes.Clear();
            routingLayer.Routes.Add(result.Route);

            winformsMap1.Refresh(winformsMap1.Overlays["RoutingOverlay"]);
        }

        private void RenderMap()
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#e6e5d1"));

            WorldMapKitWmsDesktopOverlay worldMapKitsOverlay = new WorldMapKitWmsDesktopOverlay();
            winformsMap1.Overlays.Add(worldMapKitsOverlay);

            ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(@"...\rami1\Build\JD_SAMPLE.shp");
            layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad3;
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay overlay = new LayerOverlay();
            overlay.IsBase = false;
            overlay.Layers.Add(layer);
            winformsMap1.Overlays.Add(overlay);

            ShapeFileFeatureLayer austinstreetsLayer = new ShapeFileFeatureLayer(@"...\rami1\Build\JD_SAMPLE.shp");
            austinstreetsLayer.Open();

            string[] startCoordinates = txtStartCoordinate.Text.Split(',');
            PointShape startPoint = new PointShape(double.Parse(startCoordinates[0], CultureInfo.InvariantCulture), double.Parse(startCoordinates[1], CultureInfo.InvariantCulture));
            string[] endCoordinates = txtEndCoordinate.Text.Split(',');
            PointShape endPoint = new PointShape(double.Parse(endCoordinates[0], CultureInfo.InvariantCulture), double.Parse(endCoordinates[1], CultureInfo.InvariantCulture));

            RoutingLayer routingLayer = new RoutingLayer();
            routingLayer.StartPoint = startPoint;
            routingLayer.EndPoint = endPoint;
            LayerOverlay routingOverlay = new LayerOverlay();
            routingOverlay.Layers.Add("RoutingLayer", routingLayer);
            winformsMap1.Overlays.Add("RoutingOverlay", routingOverlay);

            InMemoryFeatureLayer routingExtentLayer = new InMemoryFeatureLayer();
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColor.SimpleColors.Green));
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            routingExtentLayer.InternalFeatures.Add(new Feature(new RectangleShape(-97.815409, 30.369949, -97.657999, 30.217922)));
            routingOverlay.Layers.Add("RoutingExtentLayer", routingExtentLayer);

            winformsMap1.CurrentExtent = austinstreetsLayer.GetBoundingBox();

            winformsMap1.Refresh();
        }


        #region Component Designer generated code

        private Label lblCoordinate;
        private Button btnRoute;
        private GroupBox groupBox1;
        private Label label1;
        private TextBox txtEndCoordinate;
        private TextBox txtStartCoordinate;
        private Label label2;
        private WinformsMap winformsMap1;
        private Label label3;
        /// <summary> 
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary> 
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        /// <summary> 
        /// Required method for Designer support - do not modify 
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.lblCoordinate = new System.Windows.Forms.Label();
            this.btnRoute = new System.Windows.Forms.Button();
            this.groupBox1 = new System.Windows.Forms.GroupBox();
            this.label3 = new System.Windows.Forms.Label();
            this.txtStartCoordinate = new System.Windows.Forms.TextBox();
            this.label2 = new System.Windows.Forms.Label();
            this.txtEndCoordinate = new System.Windows.Forms.TextBox();
            this.label1 = new System.Windows.Forms.Label();
            this.winformsMap1 = new ThinkGeo.MapSuite.DesktopEdition.WinformsMap();
            this.groupBox1.SuspendLayout();
            this.SuspendLayout();
            // 
            // lblCoordinate
            // 
            this.lblCoordinate.AutoSize = true;
            this.lblCoordinate.Location = new System.Drawing.Point(204, 234);
            this.lblCoordinate.Name = "lblCoordinate";
            this.lblCoordinate.Size = new System.Drawing.Size(0, 13);
            this.lblCoordinate.TabIndex = 6;
            this.lblCoordinate.Visible = false;
            // 
            // btnRoute
            // 
            this.btnRoute.Location = new System.Drawing.Point(84, 133);
            this.btnRoute.Name = "btnRoute";
            this.btnRoute.Size = new System.Drawing.Size(139, 23);
            this.btnRoute.TabIndex = 3;
            this.btnRoute.Text = "Get The Shortest Path";
            this.btnRoute.UseVisualStyleBackColor = true;
            this.btnRoute.Click += new System.EventHandler(this.btnRoute_Click);
            // 
            // groupBox1
            // 
            this.groupBox1.Controls.Add(this.label3);
            this.groupBox1.Controls.Add(this.txtStartCoordinate);
            this.groupBox1.Controls.Add(this.label2);
            this.groupBox1.Controls.Add(this.txtEndCoordinate);
            this.groupBox1.Controls.Add(this.label1);
            this.groupBox1.Controls.Add(this.btnRoute);
            this.groupBox1.Location = new System.Drawing.Point(514, 0);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(226, 165);
            this.groupBox1.TabIndex = 8;
            this.groupBox1.TabStop = false;
            this.groupBox1.Text = "Instructions";
            // 
            // label3
            // 
            this.label3.AutoSize = true;
            this.label3.Location = new System.Drawing.Point(5, 21);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(213, 26);
            this.label3.TabIndex = 14;
            this.label3.Text = "Please click Button to get the shortest path \r\nbetween the points that are set as" +
                " below.";
            // 
            // txtStartCoordinate
            // 
            this.txtStartCoordinate.Enabled = false;
            this.txtStartCoordinate.Location = new System.Drawing.Point(85, 76);
            this.txtStartCoordinate.Name = "txtStartCoordinate";
            this.txtStartCoordinate.Size = new System.Drawing.Size(135, 20);
            this.txtStartCoordinate.TabIndex = 1;
            this.txtStartCoordinate.Text = "35.9102679453256,31.973444658283";
            // 
            // label2
            // 
            this.label2.AutoSize = true;
            this.label2.Location = new System.Drawing.Point(2, 79);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(86, 13);
            this.label2.TabIndex = 13;
            this.label2.Text = "Start Coordinate:";
            // 
            // txtEndCoordinate
            // 
            this.txtEndCoordinate.Enabled = false;
            this.txtEndCoordinate.Location = new System.Drawing.Point(85, 107);
            this.txtEndCoordinate.Name = "txtEndCoordinate";
            this.txtEndCoordinate.Size = new System.Drawing.Size(135, 20);
            this.txtEndCoordinate.TabIndex = 2;
            this.txtEndCoordinate.Text = "35.9092165193917,31.9723798213041";
            // 
            // label1
            // 
            this.label1.AutoSize = true;
            this.label1.Location = new System.Drawing.Point(2, 111);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(83, 13);
            this.label1.TabIndex = 10;
            this.label1.Text = "End Coordinate:";
            // 
            // winformsMap1
            // 
            this.winformsMap1.BackColor = System.Drawing.Color.White;
            this.winformsMap1.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.Default;
            this.winformsMap1.CurrentScale = 590591790;
            this.winformsMap1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.winformsMap1.DrawingQuality = ThinkGeo.MapSuite.Core.DrawingQuality.Default;
            this.winformsMap1.Location = new System.Drawing.Point(0, 0);
            this.winformsMap1.MapFocusMode = ThinkGeo.MapSuite.DesktopEdition.MapFocusMode.Default;
            this.winformsMap1.MapResizeMode = ThinkGeo.MapSuite.Core.MapResizeMode.PreserveScale;
            this.winformsMap1.MapUnit = ThinkGeo.MapSuite.Core.GeographyUnit.DecimalDegree;
            this.winformsMap1.MaximumScale = 80000000000000;
            this.winformsMap1.MinimumScale = 200;
            this.winformsMap1.Name = "winformsMap1";
            this.winformsMap1.Size = new System.Drawing.Size(740, 528);
            this.winformsMap1.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            this.winformsMap1.TabIndex = 9;
            this.winformsMap1.Text = "winformsMap1";
            this.winformsMap1.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            this.winformsMap1.ThreadingMode = ThinkGeo.MapSuite.DesktopEdition.MapThreadingMode.SingleThreaded;
            this.winformsMap1.ZoomLevelSnapping = ThinkGeo.MapSuite.DesktopEdition.ZoomLevelSnappingMode.Default;
            // 
            // GetShortestPathByCoordinates
            // 
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
            this.Controls.Add(this.groupBox1);
            this.Controls.Add(this.lblCoordinate);
            this.Controls.Add(this.winformsMap1);
            this.Name = "GetShortestPathByCoordinates";
            this.Size = new System.Drawing.Size(740, 528);
            this.Load += new System.EventHandler(this.UserControl_Load);
            this.groupBox1.ResumeLayout(false);
            this.groupBox1.PerformLayout();
            this.ResumeLayout(false);
            this.PerformLayout();

        }

        #endregion
    }
}
